@@ -1,9 +1,8 @@
 # -*- encoding: utf-8 -*-
 #
+# Copyright © 2016 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
-# Authors: Julien Danjou <julien@danjou.info>
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -17,21 +16,24 @@
 # under the License.
 """Time series data manipulation, better with pancetta."""
 
+import datetime
 import functools
 import logging
+import numbers
 import operator
 import re
 
+import iso8601
+import lz4
 import msgpack
 import pandas
 import six
 
+from gnocchi import utils
+
 
 LOG = logging.getLogger(__name__)
 
-AGGREGATION_METHODS = set(('mean', 'sum', 'last', 'max', 'min',
-                           'std', 'median', 'first', 'count'))
-
 
 class NoDeloreanAvailable(Exception):
     """Error raised when trying to insert a value that is too old."""
@@ -64,16 +66,9 @@ class SerializableMixin(object):
     def unserialize(cls, data):
         return cls.from_dict(msgpack.loads(data, encoding='utf-8'))
 
-    @classmethod
-    def unserialize_from_file(cls, stream):
-        return cls.from_dict(msgpack.unpack(stream, encoding='utf-8'))
-
     def serialize(self):
         return msgpack.dumps(self.to_dict())
 
-    def serialize_to_file(self, stream):
-        return msgpack.pack(self.to_dict(), stream)
-
 
 class TimeSerie(SerializableMixin):
     """A representation of series of a timestamp with a value.
@@ -82,13 +77,18 @@ class TimeSerie(SerializableMixin):
     last in the group when the TimeSerie is created or extended.
     """
 
-    def __init__(self, timestamps=None, values=None):
-        self.ts = pandas.Series(values, timestamps).groupby(
-            level=0).last().sort_index()
+    def __init__(self, ts=None):
+        if ts is None:
+            ts = pandas.Series()
+        self.ts = ts.groupby(level=0).last().sort_index()
+
+    @classmethod
+    def from_data(cls, timestamps=None, values=None):
+        return cls(pandas.Series(values, timestamps))
 
     @classmethod
     def from_tuples(cls, timestamps_values):
-        return cls(*zip(*timestamps_values))
+        return cls.from_data(*zip(*timestamps_values))
 
     def __eq__(self, other):
         return (isinstance(other, TimeSerie)
@@ -124,7 +124,8 @@ class TimeSerie(SerializableMixin):
         :param d: The dict.
         :returns: A TimeSerie object
         """
-        return cls(*cls._timestamps_and_values_from_dict(d['values']))
+        return cls.from_data(
+            *cls._timestamps_and_values_from_dict(d['values']))
 
     def to_dict(self):
         return {
@@ -136,17 +137,36 @@ class TimeSerie(SerializableMixin):
     @staticmethod
     def _serialize_time_period(value):
         if value:
-            return six.text_type(value.n) + value.rule_code
+            return value.nanos / 10e8
 
     @staticmethod
     def _round_timestamp(ts, freq):
         return pandas.Timestamp(
-            (ts.value // freq.delta.value) * freq.delta.value)
+            (pandas.Timestamp(ts).value // freq) * freq)
+
+    @staticmethod
+    def _to_offset(value):
+        if isinstance(value, numbers.Real):
+            return pandas.tseries.offsets.Nano(value * 10e8)
+        return pandas.tseries.frequencies.to_offset(value)
+
+    @property
+    def first(self):
+        try:
+            return self.ts.index[0]
+        except IndexError:
+            return
+
+    @property
+    def last(self):
+        try:
+            return self.ts.index[-1]
+        except IndexError:
+            return
 
 
 class BoundTimeSerie(TimeSerie):
-    def __init__(self, timestamps=None, values=None,
-                 block_size=None, back_window=0):
+    def __init__(self, ts=None, block_size=None, back_window=0):
         """A time serie that is limited in size.
 
         Used to represent the full-resolution buffer of incoming raw
@@ -162,14 +182,17 @@ class BoundTimeSerie(TimeSerie):
         used.
 
         """
-        super(BoundTimeSerie, self).__init__(timestamps, values)
-        if isinstance(block_size, (float, six.integer_types)):
-            self.block_size = pandas.tseries.offsets.Nano(block_size * 10e8)
-        else:
-            self.block_size = pandas.tseries.frequencies.to_offset(block_size)
+        super(BoundTimeSerie, self).__init__(ts)
+        self.block_size = self._to_offset(block_size)
         self.back_window = back_window
         self._truncate()
 
+    @classmethod
+    def from_data(cls, timestamps=None, values=None,
+                  block_size=None, back_window=0):
+        return cls(pandas.Series(values, timestamps),
+                   block_size=block_size, back_window=back_window)
+
     def __eq__(self, other):
         return (isinstance(other, BoundTimeSerie)
                 and super(BoundTimeSerie, self).__eq__(other)
@@ -210,9 +233,9 @@ class BoundTimeSerie(TimeSerie):
         :returns: A TimeSerie object
         """
         timestamps, values = cls._timestamps_and_values_from_dict(d['values'])
-        return cls(timestamps, values,
-                   block_size=d.get('block_size'),
-                   back_window=d.get('back_window'))
+        return cls.from_data(timestamps, values,
+                             block_size=d.get('block_size'),
+                             back_window=d.get('back_window'))
 
     def to_dict(self):
         basic = super(BoundTimeSerie, self).to_dict()
@@ -223,7 +246,8 @@ class BoundTimeSerie(TimeSerie):
         return basic
 
     def _first_block_timestamp(self):
-        rounded = self._round_timestamp(self.ts.index[-1], self.block_size)
+        rounded = self._round_timestamp(self.ts.index[-1],
+                                        self.block_size.delta.value)
         return rounded - (self.block_size * self.back_window)
 
     def _truncate(self):
@@ -239,16 +263,17 @@ class AggregatedTimeSerie(TimeSerie):
 
     _AGG_METHOD_PCT_RE = re.compile(r"([1-9][0-9]?)pct")
 
-    def __init__(self, timestamps=None, values=None,
-                 max_size=None,
-                 sampling=None, aggregation_method='mean'):
+    POINTS_PER_SPLIT = 14400
+
+    def __init__(self, sampling, ts=None, max_size=None,
+                 aggregation_method='mean'):
         """A time serie that is downsampled.
 
         Used to represent the downsampled timeserie for a single
         granularity/aggregation-function pair stored for a metric.
 
         """
-        super(AggregatedTimeSerie, self).__init__(timestamps, values)
+        super(AggregatedTimeSerie, self).__init__(ts)
 
         m = self._AGG_METHOD_PCT_RE.match(aggregation_method)
 
@@ -261,10 +286,55 @@ class AggregatedTimeSerie(TimeSerie):
                 raise UnknownAggregationMethod(aggregation_method)
             self.aggregation_method_func_name = aggregation_method
 
-        self.sampling = pandas.tseries.frequencies.to_offset(sampling)
+        self.sampling = self._to_offset(sampling).nanos / 10e8
         self.max_size = max_size
         self.aggregation_method = aggregation_method
 
+    @classmethod
+    def from_data(cls, sampling, timestamps=None, values=None,
+                  max_size=None, aggregation_method='mean'):
+        return cls(ts=pandas.Series(values, timestamps),
+                   max_size=max_size, sampling=sampling,
+                   aggregation_method=aggregation_method)
+
+    @classmethod
+    def get_split_key_datetime(cls, timestamp, sampling):
+        return cls._round_timestamp(
+            timestamp, freq=sampling * cls.POINTS_PER_SPLIT * 10e8)
+
+    @staticmethod
+    def _split_key_to_string(timestamp):
+        ts = timestamp.to_datetime()
+        if ts.tzinfo is None:
+            ts = ts.replace(tzinfo=iso8601.iso8601.UTC)
+        return str(utils.datetime_to_unix(ts))
+
+    @classmethod
+    def get_split_key(cls, timestamp, sampling):
+        return cls._split_key_to_string(
+            cls.get_split_key_datetime(timestamp, sampling))
+
+    def split(self):
+        groupby = self.ts.groupby(functools.partial(
+            self.get_split_key_datetime, sampling=self.sampling))
+        keys = sorted(groupby.groups.keys())
+        for i, ts in enumerate(keys):
+            if i + 1 == len(keys):
+                yield self._split_key_to_string(ts), TimeSerie(self.ts[ts:])
+            elif i + 1 < len(keys):
+                t = self.ts[ts:keys[i + 1]]
+                del t[t.index[-1]]
+                yield self._split_key_to_string(ts), TimeSerie(t)
+
+    @classmethod
+    def from_timeseries(cls, timeseries, sampling, max_size=None,
+                        aggregation_method='mean'):
+        ts = pandas.Series()
+        for t in timeseries:
+            ts = ts.combine_first(t.ts)
+        return cls(ts=ts, sampling=sampling, max_size=max_size,
+                   aggregation_method=aggregation_method)
+
     def __eq__(self, other):
         return (isinstance(other, AggregatedTimeSerie)
                 and super(AggregatedTimeSerie, self).__eq__(other)
@@ -272,6 +342,15 @@ class AggregatedTimeSerie(TimeSerie):
                 and self.sampling == other.sampling
                 and self.aggregation_method == other.aggregation_method)
 
+    def __repr__(self):
+        return "<%s 0x%x sampling=%fs max_size=%s agg_method=%s>" % (
+            self.__class__.__name__,
+            id(self),
+            self.sampling,
+            self.max_size,
+            self.aggregation_method,
+        )
+
     @classmethod
     def from_dict(cls, d):
         """Build a time series from a dict.
@@ -281,20 +360,58 @@ class AggregatedTimeSerie(TimeSerie):
         :param d: The dict.
         :returns: A TimeSerie object
         """
-        timestamps, values = cls._timestamps_and_values_from_dict(d['values'])
-        return cls(timestamps, values,
-                   max_size=d.get('max_size'),
-                   sampling=d.get('sampling'),
-                   aggregation_method=d.get('aggregation_method', 'mean'))
+        sampling = d.get('sampling')
+        prev_timestamp = pandas.Timestamp(d.get('first_timestamp') * 10e8)
+        timestamps = []
+        for delta in d.get('timestamps'):
+            prev_timestamp = datetime.timedelta(
+                seconds=delta * sampling) + prev_timestamp
+            timestamps.append(prev_timestamp)
+
+        return cls.from_data(
+            timestamps=timestamps,
+            values=d.get('values'),
+            max_size=d.get('max_size'),
+            sampling=sampling,
+            aggregation_method=d.get('aggregation_method', 'mean'))
 
     def to_dict(self):
-        d = super(AggregatedTimeSerie, self).to_dict()
-        d.update({
+        if self.ts.empty:
+            timestamps = []
+            values = []
+            first_timestamp = 0
+        else:
+            first_timestamp = float(
+                self.get_split_key(self.ts.index[0], self.sampling))
+            timestamps = []
+            prev_timestamp = pandas.Timestamp(
+                first_timestamp * 10e8).to_pydatetime()
+            # Use double delta encoding for timestamps
+            for i in self.ts.index:
+                # Convert to pydatetime because it's faster to compute than
+                # Pandas' objects
+                asdt = i.to_pydatetime()
+                timestamps.append(
+                    int((asdt - prev_timestamp).total_seconds()
+                        / self.sampling))
+                prev_timestamp = asdt
+            values = self.ts.values.tolist()
+
+        return {
+            'first_timestamp': first_timestamp,
             'aggregation_method': self.aggregation_method,
             'max_size': self.max_size,
-            'sampling': self._serialize_time_period(self.sampling),
-        })
-        return d
+            'sampling': self.sampling,
+            'timestamps': timestamps,
+            'values': values,
+        }
+
+    @classmethod
+    def unserialize(cls, data):
+        return cls.from_dict(msgpack.loads(lz4.loads(data), encoding='utf-8'))
+
+    def serialize(self):
+        return lz4.dumps(msgpack.dumps(self.to_dict()))
 
     def _truncate(self):
         """Truncate the timeserie."""
@@ -303,20 +420,41 @@ class AggregatedTimeSerie(TimeSerie):
             self.ts = self.ts.dropna()[-self.max_size:]
 
     def _resample(self, after):
-        if self.sampling:
-            # Group by the sampling, and then apply the aggregation method on
-            # the points after `after'
-            groupedby = self.ts[after:].groupby(
-                functools.partial(self._round_timestamp,
-                                  freq=self.sampling))
-            agg_func = getattr(groupedby, self.aggregation_method_func_name)
-            if self.aggregation_method_func_name == 'quantile':
-                aggregated = agg_func(self.q)
-            else:
-                aggregated = agg_func()
-            # Now combine the result with the rest of the point – everything
-            # that is before `after'
-            self.ts = aggregated.combine_first(self.ts[:after][:-1])
+        # Group by the sampling, and then apply the aggregation method on
+        # the points after `after'
+        groupedby = self.ts[after:].groupby(
+            functools.partial(self._round_timestamp,
+                              freq=self.sampling * 10e8))
+        agg_func = getattr(groupedby, self.aggregation_method_func_name)
+        if self.aggregation_method_func_name == 'quantile':
+            aggregated = agg_func(self.q)
+        else:
+            aggregated = agg_func()
+        # Now combine the result with the rest of the point – everything
+        # that is before `after'
+        self.ts = aggregated.combine_first(self.ts[:after][:-1])
+
+    def fetch(self, from_timestamp=None, to_timestamp=None):
+        """Fetch aggregated time value.
+
+        Returns a sorted list of tuples (timestamp, granularity, value).
+        """
+        # Round timestamp to our granularity so we're sure that if e.g. 17:02
+        # is requested and we have points for 17:00 and 17:05 in a 5min
+        # granularity, we do return the 17:00 point and not nothing
+        if from_timestamp is None:
+            from_ = None
+        else:
+            from_ = self._round_timestamp(from_timestamp, self.sampling * 10e8)
+        points = self[from_:to_timestamp]
+        try:
+            # Do not include stop timestamp
+            del points[to_timestamp]
+        except KeyError:
+            pass
+        return [(timestamp, self.sampling, value)
+                for timestamp, value
+                in six.iteritems(points)]
 
     def update(self, ts):
         if ts.ts.empty:
@@ -341,80 +479,6 @@ class AggregatedTimeSerie(TimeSerie):
         self._resample(first_timestamp)
         self._truncate()
 
-
-class TimeSerieArchive(SerializableMixin):
-
-    def __init__(self, agg_timeseries):
-        """A raw data buffer and a collection of downsampled timeseries.
-
-        Used to represent the set of AggregatedTimeSeries for the range of
-        granularities supported for a metric (for a particular aggregation
-        function).
-
-        """
-        self.agg_timeseries = sorted(agg_timeseries,
-                                     key=operator.attrgetter("sampling"))
-
-    @property
-    def max_block_size(self):
-        return max(agg.sampling for agg in self.agg_timeseries)
-
-    @classmethod
-    def from_definitions(cls, definitions, aggregation_method='mean'):
-        """Create a new collection of archived time series.
-
-        :param definition: A list of tuple (sampling, max_size)
-        :param aggregation_method: Aggregation function to use.
-        """
-        # Limit the main timeserie to a timespan mapping
-        return cls(
-            [AggregatedTimeSerie(
-                max_size=size,
-                sampling=pandas.tseries.offsets.Nano(sampling * 10e8),
-                aggregation_method=aggregation_method)
-             for sampling, size in definitions]
-        )
-
-    def fetch(self, from_timestamp=None, to_timestamp=None,
-              timeserie_filter=None):
-        """Fetch aggregated time value.
-
-        Returns a sorted list of tuples (timestamp, granularity, value).
-        """
-        result = []
-        end_timestamp = to_timestamp
-        for ts in reversed(self.agg_timeseries):
-            if timeserie_filter and not timeserie_filter(ts):
-                continue
-            granularity = ts.sampling.nanos / 1000000000.0
-            points = ts[from_timestamp:to_timestamp]
-            try:
-                # Do not include stop timestamp
-                del points[end_timestamp]
-            except KeyError:
-                pass
-            result.extend([(timestamp, granularity, value)
-                           for timestamp, value
-                           in six.iteritems(points)])
-        return result
-
-    def __eq__(self, other):
-        return (isinstance(other, TimeSerieArchive)
-                and self.agg_timeseries == other.agg_timeseries)
-
-    def update(self, timeserie):
-        for agg in self.agg_timeseries:
-            agg.update(timeserie)
-
-    def to_dict(self):
-        return {
-            "archives": [ts.to_dict() for ts in self.agg_timeseries],
-        }
-
-    @classmethod
-    def from_dict(cls, d):
-        return cls([AggregatedTimeSerie.from_dict(a) for a in d['archives']])
-
     @staticmethod
     def aggregated(timeseries, from_timestamp=None, to_timestamp=None,
                    aggregation='mean', needed_percent_of_overlap=100.0):
@@ -426,16 +490,8 @@ class TimeSerieArchive(SerializableMixin):
         if not timeseries:
             return []
 
-        granularities = [set(ts.sampling for ts in timeserie.agg_timeseries)
-                         for timeserie in timeseries]
-        granularities = granularities[0].intersection(*granularities[1:])
-        if len(granularities) == 0:
-            raise UnAggregableTimeseries('No granularity match')
-
         for timeserie in timeseries:
-            timeserie_raw = timeserie.fetch(
-                from_timestamp, to_timestamp,
-                lambda ts: ts.sampling in granularities)
+            timeserie_raw = timeserie.fetch(from_timestamp, to_timestamp)
 
             if timeserie_raw:
                 dataframe = pandas.DataFrame(timeserie_raw, columns=columns)
@@ -445,20 +501,37 @@ class TimeSerieArchive(SerializableMixin):
         if not dataframes:
             return []
 
+        number_of_distinct_datasource = len(timeseries) / len(
+            set(ts.sampling for ts in timeseries)
+        )
+
         grouped = pandas.concat(dataframes).groupby(level=index)
         left_boundary_ts = None
         right_boundary_ts = None
         maybe_next_timestamp_is_left_boundary = False
+
+        left_holes = 0
+        right_holes = 0
         holes = 0
         for (timestamp, __), group in grouped:
-            if group.count()['value'] != len(timeseries):
+            if group.count()['value'] != number_of_distinct_datasource:
                 maybe_next_timestamp_is_left_boundary = True
-                holes += 1
+                if left_boundary_ts is not None:
+                    right_holes += 1
+                else:
+                    left_holes += 1
             elif maybe_next_timestamp_is_left_boundary:
                 left_boundary_ts = timestamp
                 maybe_next_timestamp_is_left_boundary = False
             else:
                 right_boundary_ts = timestamp
+                holes += right_holes
+                right_holes = 0
+
+        if to_timestamp is not None:
+            holes += left_holes
+        if from_timestamp is not None:
+            holes += right_holes
 
         if to_timestamp is not None or from_timestamp is not None:
             maximum = len(grouped)
@@ -469,7 +542,7 @@ class TimeSerieArchive(SerializableMixin):
                     'Less than %f%% of datapoints overlap in this '
                     'timespan (%.2f%%)' % (needed_percent_of_overlap,
                                            percent_of_overlap))
-        elif (needed_percent_of_overlap > 0 and
+        if (needed_percent_of_overlap > 0 and
                 (right_boundary_ts == left_boundary_ts or
                  (right_boundary_ts is None
                   and maybe_next_timestamp_is_left_boundary))):
@@ -486,7 +559,7 @@ class TimeSerieArchive(SerializableMixin):
 
         # NOTE(sileht): this call the aggregation method on already
         # aggregated values, for some kind of aggregation this can
-        # result can looks wierd, but this is the best we can do
+        # result can looks weird, but this is the best we can do
         # because we don't have anymore the raw datapoints in those case.
         # FIXME(sileht): so should we bailout is case of stddev, percentile
         # and median?
@@ -506,112 +579,70 @@ class TimeSerieArchive(SerializableMixin):
                 for __, timestamp, granularity, value in points]
 
 
-import argparse
-import datetime
+class TimeSerieArchive(SerializableMixin):
 
-from oslo_utils import timeutils
-import prettytable
+    def __init__(self, agg_timeseries):
+        """A raw data buffer and a collection of downsampled timeseries.
 
+        Used to represent the set of AggregatedTimeSeries for the range of
+        granularities supported for a metric (for a particular aggregation
+        function).
 
-def _definition(value):
-    result = value.split(",")
-    if len(result) != 2:
-        raise ValueError("Format is: seconds,points")
-    return int(result[0]), int(result[1])
+        """
+        self.agg_timeseries = sorted(agg_timeseries,
+                                     key=operator.attrgetter("sampling"))
 
+    @classmethod
+    def from_definitions(cls, definitions, aggregation_method='mean'):
+        """Create a new collection of archived time series.
 
-def create_archive_file():
-    parser = argparse.ArgumentParser(
-        description="Create a Carbonara file",
-    )
-    parser.add_argument("--aggregation-method",
-                        type=six.text_type,
-                        default="mean",
-                        choices=AGGREGATION_METHODS,
-                        help="aggregation method to use")
-    parser.add_argument("--back-window",
-                        type=int,
-                        default=0,
-                        help="back window to keep")
-    parser.add_argument("definition",
-                        type=_definition,
-                        nargs='+',
-                        help="archive definition as granularity,points")
-    parser.add_argument("filename",
-                        nargs=1,
-                        type=argparse.FileType(mode="wb"),
-                        help="File name to create")
-    args = parser.parse_args()
-    ts = TimeSerieArchive.from_definitions(args.definition,
-                                           args.aggregation_method)
-    args.filename[0].write(ts.serialize())
+        :param definition: A list of tuple (sampling, max_size)
+        :param aggregation_method: Aggregation function to use.
+        """
+        # Limit the main timeserie to a timespan mapping
+        return cls(
+            [AggregatedTimeSerie(
+                max_size=size,
+                sampling=sampling,
+                aggregation_method=aggregation_method)
+             for sampling, size in definitions]
+        )
 
+    def fetch(self, from_timestamp=None, to_timestamp=None,
+              timeserie_filter=None):
+        """Fetch aggregated time value.
 
-def dump_archive_file():
-    parser = argparse.ArgumentParser(
-        description="Dump a Carbonara file",
-    )
-    parser.add_argument("filename",
-                        nargs=1,
-                        type=argparse.FileType(mode="rb"),
-                        help="File name to read")
-    args = parser.parse_args()
+        Returns a sorted list of tuples (timestamp, granularity, value).
+        """
+        result = []
+        end_timestamp = to_timestamp
+        for ts in reversed(self.agg_timeseries):
+            if timeserie_filter and not timeserie_filter(ts):
+                continue
+            points = ts[from_timestamp:to_timestamp]
+            try:
+                # Do not include stop timestamp
+                del points[end_timestamp]
+            except KeyError:
+                pass
+            result.extend([(timestamp, ts.sampling, value)
+                           for timestamp, value
+                           in six.iteritems(points)])
+        return result
 
-    ts = TimeSerieArchive.unserialize_from_file(args.filename[0])
+    def update(self, timeserie):
+        for agg in self.agg_timeseries:
+            agg.update(timeserie)
 
-    print("Aggregation method: %s"
-          % (ts.agg_timeseries[0].aggregation_method))
+    def to_dict(self):
+        return {
+            "archives": [ts.to_dict() for ts in self.agg_timeseries],
+        }
 
-    print("Number of aggregated timeseries: %d" % len(ts.agg_timeseries))
+    def __eq__(self, other):
+        return (isinstance(other, TimeSerieArchive)
+                and self.agg_timeseries == other.agg_timeseries)
 
-    for idx, agg_ts in enumerate(ts.agg_timeseries):
-        sampling = agg_ts.sampling.nanos / 1000000000
-        timespan = datetime.timedelta(seconds=sampling * agg_ts.max_size)
-        print("\nAggregated timeserie #%d: %ds × %d = %s"
-              % (idx + 1, sampling, agg_ts.max_size, timespan))
-        print("Number of measures: %d" % len(agg_ts))
-        table = prettytable.PrettyTable(("Timestamp", "Value"))
-        for k, v in agg_ts.ts.iteritems():
-            table.add_row((k, v))
-        print(table.get_string())
-
-
-def _timestamp_value(value):
-    result = value.split(",")
-    if len(result) != 2:
-        raise ValueError("Format is: timestamp,value")
-    try:
-        timestamp = float(result[0])
-    except (ValueError, TypeError):
-        timestamp = timeutils.normalize_time(
-            timeutils.parse_isotime(result[0]))
-    else:
-        timestamp = datetime.datetime.utcfromtimestamp(timestamp)
-
-    return timestamp, float(result[1])
-
-
-def update_archive_file():
-    parser = argparse.ArgumentParser(
-        description="Insert values in a Carbonara file",
-    )
-    parser.add_argument("timestamp,value",
-                        nargs='+',
-                        type=_timestamp_value,
-                        help="Timestamp and value to set")
-    parser.add_argument("filename",
-                        nargs=1,
-                        type=argparse.FileType(mode="rb+"),
-                        help="File name to update")
-    args = parser.parse_args()
-
-    ts = TimeSerieArchive.unserialize_from_file(args.filename[0])
-
-    try:
-        ts.update(TimeSerie.from_tuples(getattr(args, 'timestamp,value')))
-    except Exception as e:
-        print("E: %s: %s" % (e.__class__.__name__, e))
-        return 1
-
-    args.filename[0].seek(0)
-    ts.serialize_to_file(args.filename[0])
+    @classmethod
+    def from_dict(cls, d):
+        return cls([AggregatedTimeSerie.from_dict(a) for a in d['archives']])
