@@ -1,4 +1,5 @@
 # Copyright (c) 2013 Mirantis Inc.
+# Copyright (c) 2015 Red Hat
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,11 +19,11 @@ import signal
 import sys
 import time
 
+from oslo_config import cfg
 from oslo_utils import timeutils
 import retrying
 
 from gnocchi import indexer
-from gnocchi.indexer import sqlalchemy as sql_db
 from gnocchi.rest import app
 from gnocchi import service
 from gnocchi import statsd as statsd_service
@@ -32,11 +33,24 @@ from gnocchi import storage
 LOG = logging.getLogger(__name__)
 
 
-def storage_dbsync():
-    conf = service.prepare_service()
-    index = sql_db.SQLAlchemyIndexer(conf)
-    index.connect()
-    index.upgrade()
+def upgrade():
+    conf = cfg.ConfigOpts()
+    conf.register_cli_opts([
+        cfg.BoolOpt("skip-index", default=False,
+                    help="Skip index upgrade."),
+        cfg.BoolOpt("skip-storage", default=False,
+                    help="Skip storage upgrade.")
+    ])
+    conf = service.prepare_service(conf=conf)
+    if not conf.skip_index:
+        index = indexer.get_driver(conf)
+        index.connect()
+        LOG.info("Upgrading indexer %s" % index)
+        index.upgrade()
+    if not conf.skip_storage:
+        s = storage.get_driver(conf)
+        LOG.info("Upgrading storage %s" % s)
+        s.upgrade(index)
 
 
 def api():
@@ -48,10 +62,11 @@ def statsd():
 
 
 class MetricProcessBase(multiprocessing.Process):
-    def __init__(self, conf, startup_delay=0, interval_delay=0):
+    def __init__(self, conf, worker_id=0, interval_delay=0):
         super(MetricProcessBase, self).__init__()
         self.conf = conf
-        self.startup_delay = startup_delay
+        self.worker_id = worker_id
+        self.startup_delay = worker_id
         self.interval_delay = interval_delay
 
     # Retry with exponential backoff for up to 5 minutes
@@ -60,6 +75,7 @@ class MetricProcessBase(multiprocessing.Process):
                     stop_max_delay=300000)
     def _configure(self):
         self.store = storage.get_driver(self.conf)
+        self.store.partition = self.worker_id
         self.index = indexer.get_driver(self.conf)
         self.index.connect()
 
@@ -71,65 +87,95 @@ class MetricProcessBase(multiprocessing.Process):
         while True:
             try:
                 with timeutils.StopWatch() as timer:
-                    self.store.process_background_tasks(self.index)
+                    self._run_job()
                     time.sleep(max(0, self.interval_delay - timer.elapsed()))
             except KeyboardInterrupt:
                 # Ignore KeyboardInterrupt so parent handler can kill
                 # all children.
                 pass
 
+    @staticmethod
+    def _run_job():
+        raise NotImplementedError
+
 
 class MetricReporting(MetricProcessBase):
+    def __init__(self, conf, worker_id=0, interval_delay=0, queues=None):
+        super(MetricReporting, self).__init__(conf, worker_id, interval_delay)
+        self.queues = queues
+
     def _run_job(self):
         try:
-            report = self.store.measures_report()
+            report = self.store.measures_report(details=False)
+            if self.queues:
+                block_size = max(16, min(256, len(report) // len(self.queues)))
+                for queue in self.queues:
+                    queue.put(block_size)
             LOG.info("Metricd reporting: %d measurements bundles across %d "
-                     "metrics wait to be processed." %
-                     (sum(report.values()), len(report)))
+                     "metrics wait to be processed.",
+                     report['summary']['measures'],
+                     report['summary']['metrics'])
         except Exception:
             LOG.error("Unexpected error during pending measures reporting",
                       exc_info=True)
 
 
 class MetricProcessor(MetricProcessBase):
+    def __init__(self, conf, worker_id=0, interval_delay=0, queue=None):
+        super(MetricProcessor, self).__init__(conf, worker_id, interval_delay)
+        self.queue = queue
+        self.block_size = 128
+
     def _run_job(self):
-            LOG.debug("Processing new measures")
-            try:
-                self.store.process_measures(self.index)
-            except Exception:
-                LOG.error("Unexpected error during measures processing",
-                          exc_info=True)
+        try:
+            if self.queue:
+                while not self.queue.empty():
+                    self.block_size = self.queue.get()
+            self.store.process_background_tasks(self.index, self.block_size)
+        except Exception:
+            LOG.error("Unexpected error during measures processing",
+                      exc_info=True)
 
 
 def metricd():
     conf = service.prepare_service()
+    if (conf.storage.metric_reporting_delay <
+            conf.storage.metric_processing_delay):
+        LOG.error("Metric reporting must run less frequently then processing")
+        sys.exit(0)
 
     signal.signal(signal.SIGTERM, _metricd_terminate)
 
     try:
-        metric_report = MetricReporting(
-            conf, 0, conf.storage.metric_reporting_delay)
-        metric_report.start()
-
-        workers = [metric_report]
+        queues = []
+        workers = []
         for worker in range(conf.metricd.workers):
+            queue = multiprocessing.Queue()
             metric_worker = MetricProcessor(
-                conf, worker, conf.storage.metric_processing_delay)
+                conf, worker, conf.storage.metric_processing_delay, queue)
             metric_worker.start()
+            queues.append(queue)
             workers.append(metric_worker)
 
+        metric_report = MetricReporting(
+            conf, 0, conf.storage.metric_reporting_delay, queues)
+        metric_report.start()
+        workers.append(metric_report)
+
         for worker in workers:
             worker.join()
     except KeyboardInterrupt:
-        _metricd_cleanup(workers)
+        _metricd_cleanup(workers, queues)
         sys.exit(0)
     except Exception:
-        LOG.warn("exiting", exc_info=True)
+        LOG.warning("exiting", exc_info=True)
         _metricd_cleanup(workers)
         sys.exit(1)
 
 
-def _metricd_cleanup(workers):
+def _metricd_cleanup(workers, queues):
+    for queue in queues:
+        queue.close()
     for worker in workers:
         worker.terminate()
     for worker in workers:
