@@ -13,9 +13,10 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
+from collections import defaultdict
 import contextlib
 import datetime
+import itertools
 import logging
 import uuid
 
@@ -28,8 +29,13 @@ from gnocchi.storage import _carbonara
 
 LOG = logging.getLogger(__name__)
 
-# NOTE(sileht): rados module is not available on pypi
-rados = importutils.try_import('rados')
+for RADOS_MODULE_NAME in ('cradox', 'rados'):
+    rados = importutils.try_import(RADOS_MODULE_NAME)
+    if rados is not None:
+        break
+else:
+    RADOS_MODULE_NAME = None
+
 if rados is not None and hasattr(rados, 'run_in_thread'):
     rados.run_in_thread = lambda target, args, timeout=None: target(*args)
     LOG.info("rados.run_in_thread is monkeypatched.")
@@ -40,10 +46,8 @@ OPTS = [
                default='gnocchi',
                help='Ceph pool name to use.'),
     cfg.StrOpt('ceph_username',
-               default=None,
                help='Ceph username (ie: client.admin).'),
     cfg.StrOpt('ceph_keyring',
-               default=None,
                help='Ceph keyring path.'),
     cfg.StrOpt('ceph_conffile',
                default='/etc/ceph/ceph.conf',
@@ -52,14 +56,20 @@ OPTS = [
 
 
 class CephStorage(_carbonara.CarbonaraBasedStorage):
+
     def __init__(self, conf):
         super(CephStorage, self).__init__(conf)
         self.pool = conf.ceph_pool
-        self._lock = _carbonara.CarbonaraBasedStorageToozLock(conf)
         options = {}
         if conf.ceph_keyring:
             options['keyring'] = conf.ceph_keyring
 
+        if not rados:
+            raise ImportError("No module named 'rados' nor 'cradox'")
+
+        LOG.info("Ceph storage backend use '%s' python library" %
+                 RADOS_MODULE_NAME)
+
         # NOTE(sileht): librados handles reconnection itself,
         # by default if a call timeout (30sec), it raises
         # a rados.Timeout exception, and librados
@@ -69,9 +79,6 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
                                  conf=options)
         self.rados.connect()
 
-    def stop(self):
-        self._lock.stop()
-
     def _store_measures(self, metric, data):
         # NOTE(sileht): list all objects in a pool is too slow with
         # many objects (2min for 20000 objects in 50osds cluster),
@@ -87,24 +94,53 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             ioctx.write_full(name, data)
             ioctx.set_xattr(self.MEASURE_PREFIX, name, "")
 
-    def _list_object_names_to_process(self, ioctx, prefix=None):
+    def _build_report(self, details):
+        with self._get_ioctx() as ioctx:
+            try:
+                xattrs = ioctx.get_xattrs(self.MEASURE_PREFIX)
+            except rados.ObjectNotFound:
+                return 0, 0, {} if details else None
+        metrics = set()
+        count = 0
+        metric_details = defaultdict(int)
+        for name, __ in xattrs:
+            count += 1
+            metric = name.split("_")[1]
+            metrics.add(metric)
+            if details:
+                metric_details[metric] += 1
+        return len(metrics), count, metric_details if details else None
+
+    def _list_object_names_to_process(self, ioctx, prefix):
         try:
             xattrs = ioctx.get_xattrs(self.MEASURE_PREFIX)
         except rados.ObjectNotFound:
-            return []
-        return set(name for name, __ in xattrs
-                   if prefix is None or name.startswith(prefix))
+            return ()
+        return (name for name, __ in xattrs if name.startswith(prefix))
 
     def _pending_measures_to_process_count(self, metric_id):
         with self._get_ioctx() as ioctx:
             object_prefix = self.MEASURE_PREFIX + "_" + str(metric_id)
-            return len(self._list_object_names_to_process(ioctx,
-                                                          object_prefix))
+            return len(list(self._list_object_names_to_process(ioctx,
+                                                               object_prefix)))
 
-    def _list_metric_with_measures_to_process(self):
+    def _list_metric_with_measures_to_process(self, block_size, full=False):
         with self._get_ioctx() as ioctx:
-            return [name.split("_")[1] for name in
-                    self._list_object_names_to_process(ioctx)]
+            try:
+                xattrs = ioctx.get_xattrs(self.MEASURE_PREFIX)
+            except rados.ObjectNotFound:
+                return []
+        metrics = set()
+        if full:
+            objs_it = xattrs
+        else:
+            objs_it = itertools.islice(
+                xattrs, block_size * self.partition, None)
+        for name, __ in objs_it:
+            metrics.add(name.split("_")[1])
+            if full is False and len(metrics) >= block_size:
+                break
+        return metrics
 
     def _delete_unprocessed_measures_for_metric_id(self, metric_id):
         with self._get_ioctx() as ctx:
@@ -114,17 +150,17 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             for n in object_names:
                 try:
                     ctx.rm_xattr(self.MEASURE_PREFIX, n)
-                    ctx.remove_object(n)
                 except rados.ObjectNotFound:
                     # Another worker may have removed it, don't worry.
                     pass
+                ctx.aio_remove(n)
 
     @contextlib.contextmanager
     def _process_measure_for_metric(self, metric):
         with self._get_ioctx() as ctx:
             object_prefix = self.MEASURE_PREFIX + "_" + str(metric.id)
-            object_names = self._list_object_names_to_process(ctx,
-                                                              object_prefix)
+            object_names = list(self._list_object_names_to_process(
+                ctx, object_prefix))
 
             measures = []
             for n in object_names:
@@ -136,73 +172,100 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             # Now clean objects and xattrs
             for n in object_names:
                 ctx.rm_xattr(self.MEASURE_PREFIX, n)
-                ctx.remove_object(n)
+                ctx.aio_remove(n)
 
     def _get_ioctx(self):
         return self.rados.open_ioctx(self.pool)
 
     @staticmethod
-    def _get_object_name(metric, lock_name):
-        return str("gnocchi_%s_%s" % (metric.id, lock_name))
+    def _get_object_name(metric, timestamp_key, aggregation, granularity):
+        return str("gnocchi_%s_%s_%s_%s" % (
+            metric.id, timestamp_key, aggregation, granularity))
 
     @staticmethod
     def _object_exists(ioctx, name):
         try:
-            size, mtime = ioctx.stat(name)
-            # NOTE(sileht): the object have been created by
-            # the lock code
-            return size > 0
+            ioctx.stat(name)
+            return True
         except rados.ObjectNotFound:
             return False
 
     def _create_metric(self, metric):
-        name = self._get_object_name(metric, 'container')
+        name = "gnocchi_%s_container" % metric.id
         with self._get_ioctx() as ioctx:
             if self._object_exists(ioctx, name):
                 raise storage.MetricAlreadyExists(metric)
             else:
                 ioctx.write_full(name, "metric created")
 
-    def _store_metric_measures(self, metric, aggregation, data):
-        name = self._get_object_name(metric, aggregation)
+    def _store_metric_measures(self, metric, timestamp_key,
+                               aggregation, granularity, data):
+        name = self._get_object_name(metric, timestamp_key,
+                                     aggregation, granularity)
         with self._get_ioctx() as ioctx:
             ioctx.write_full(name, data)
+            ioctx.set_xattr("gnocchi_%s_container" % metric.id, name, "")
+
+    def _delete_metric_measures(self, metric, timestamp_key, aggregation,
+                                granularity):
+        name = self._get_object_name(metric, timestamp_key,
+                                     aggregation, granularity)
+        with self._get_ioctx() as ioctx:
+            ioctx.rm_xattr("gnocchi_%s_container" % metric.id, name)
+            ioctx.remove_object(name)
 
     def _delete_metric(self, metric):
         with self._get_ioctx() as ioctx:
+            try:
+                xattrs = ioctx.get_xattrs("gnocchi_%s_container" % metric.id)
+            except rados.ObjectNotFound:
+                pass
+            else:
+                for xattr, _ in xattrs:
+                    ioctx.aio_remove(xattr)
             for name in ('container', 'none'):
-                name = self._get_object_name(metric, name)
-                try:
-                    ioctx.remove_object(name)
-                except rados.ObjectNotFound:
-                    # Maybe it never got measures
-                    pass
-            for aggregation in metric.archive_policy.aggregation_methods:
-                name = self._get_object_name(metric, aggregation)
-                try:
-                    ioctx.remove_object(name)
-                except rados.ObjectNotFound:
-                    pass
+                ioctx.aio_remove("gnocchi_%s_%s" % (metric.id, name))
 
-    def _get_measures(self, metric, aggregation):
+    def _get_measures(self, metric, timestamp_key, aggregation, granularity):
         try:
             with self._get_ioctx() as ioctx:
-                name = self._get_object_name(metric, aggregation)
-                content = self._get_object_content(ioctx, name)
-                if len(content) == 0:
-                    # NOTE(sileht: the object have been created by
-                    # the lock code
-                    raise rados.ObjectNotFound
-                else:
-                    return content
+                name = self._get_object_name(metric, timestamp_key,
+                                             aggregation, granularity)
+                return self._get_object_content(ioctx, name)
         except rados.ObjectNotFound:
-            name = self._get_object_name(metric, 'container')
             with self._get_ioctx() as ioctx:
-                if self._object_exists(ioctx, name):
+                if self._object_exists(
+                        ioctx, "gnocchi_%s_container" % metric.id):
                     raise storage.AggregationDoesNotExist(metric, aggregation)
                 else:
                     raise storage.MetricDoesNotExist(metric)
 
+    def _list_split_keys_for_metric(self, metric, aggregation, granularity):
+        with self._get_ioctx() as ioctx:
+            try:
+                xattrs = ioctx.get_xattrs("gnocchi_%s_container" % metric.id)
+            except rados.ObjectNotFound:
+                raise storage.MetricDoesNotExist(metric)
+            keys = []
+            for xattr, value in xattrs:
+                _, metric_id, key, agg, g = xattr.split('_', 4)
+                if aggregation == agg and granularity == float(g):
+                    keys.append(key)
+
+        return keys
+
+    def _get_unaggregated_timeserie(self, metric):
+        try:
+            with self._get_ioctx() as ioctx:
+                return self._get_object_content(
+                    ioctx, "gnocchi_%s_none" % metric.id)
+        except rados.ObjectNotFound:
+            raise storage.MetricDoesNotExist(metric)
+
+    def _store_unaggregated_timeserie(self, metric, data):
+        with self._get_ioctx() as ioctx:
+            ioctx.write_full("gnocchi_%s_none" % metric.id, data)
+
     @staticmethod
     def _get_object_content(ioctx, name):
         offset = 0
@@ -214,3 +277,25 @@ class CephStorage(_carbonara.CarbonaraBasedStorage):
             content += data
             offset += len(data)
         return content
+
+    # The following methods deal with Gnocchi <= 1.3 archives
+    def _get_metric_archive(self, metric, aggregation):
+        """Retrieve data in the place we used to store TimeSerieArchive."""
+        try:
+            with self._get_ioctx() as ioctx:
+                return self._get_object_content(
+                    ioctx, str("gnocchi_%s_%s" % (metric.id, aggregation)))
+        except rados.ObjectNotFound:
+            raise storage.AggregationDoesNotExist(metric, aggregation)
+
+    def _store_metric_archive(self, metric, aggregation, data):
+        """Stores data in the place we used to store TimeSerieArchive."""
+        with self._get_ioctx() as ioctx:
+            ioctx.write_full(
+                str("gnocchi_%s_%s" % (metric.id, aggregation)), data)
+
+    def _delete_metric_archives(self, metric):
+        with self._get_ioctx() as ioctx:
+            for aggregation in metric.archive_policy.aggregation_methods:
+                ioctx.aio_remove(
+                    str("gnocchi_%s_%s" % (metric.id, aggregation)))
