@@ -17,9 +17,12 @@ import datetime
 import uuid
 
 import mock
+from oslo_utils import timeutils
+from oslotest import base
 import six.moves
 
 from gnocchi import storage
+from gnocchi.storage import _carbonara
 from gnocchi.storage import null
 from gnocchi.tests import base as tests_base
 from gnocchi import utils
@@ -29,8 +32,15 @@ class TestStorageDriver(tests_base.TestCase):
     def setUp(self):
         super(TestStorageDriver, self).setUp()
         # A lot of tests wants a metric, create one
-        self.metric = storage.Metric(uuid.uuid4(),
-                                     self.archive_policies['low'])
+        self.metric, __ = self._create_metric()
+
+    def _create_metric(self, archive_policy_name="low"):
+        m = storage.Metric(uuid.uuid4(),
+                           self.archive_policies[archive_policy_name])
+        m_sql = self.index.create_metric(m.id, str(uuid.uuid4()),
+                                         str(uuid.uuid4()),
+                                         archive_policy_name)
+        return m, m_sql
 
     def test_get_driver(self):
         self.conf.set_override('driver', 'null', 'storage')
@@ -39,35 +49,22 @@ class TestStorageDriver(tests_base.TestCase):
 
     @mock.patch('gnocchi.storage._carbonara.LOG')
     def test_corrupted_data(self, logger):
+        if not isinstance(self.storage, _carbonara.CarbonaraBasedStorage):
+            self.skipTest("This driver is not based on Carbonara")
+
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
         ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
 
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 13, 0, 1), 1),
         ])
-        with mock.patch.object(self.index, 'get_metrics',
-                               return_value=[self.metric]):
-            with mock.patch('gnocchi.carbonara.msgpack.unpack',
+        with mock.patch('gnocchi.carbonara.msgpack.unpack',
+                        side_effect=ValueError("boom!")):
+            with mock.patch('gnocchi.carbonara.msgpack.loads',
                             side_effect=ValueError("boom!")):
-                with mock.patch('gnocchi.carbonara.msgpack.loads',
-                                side_effect=ValueError("boom!")):
-                    self.storage.process_background_tasks(self.index, True)
-
-        expected_calls = [
-            mock.call.debug('Processing measures for %s' % self.metric.id),
-            mock.call.debug('Processing measures for %s' % self.metric.id),
-        ]
-        aggs = ["none"] + self.conf.archive_policy.default_aggregation_methods
-        for agg in aggs:
-            expected_calls.append(mock.call.error(
-                'Data are corrupted for metric %s and aggregation %s, '
-                'recreating an empty timeserie.' % (self.metric.id, agg)))
-
-        logger.assert_has_calls(expected_calls, any_order=True)
+                self.storage.process_background_tasks(self.index, sync=True)
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 1),
@@ -79,64 +76,93 @@ class TestStorageDriver(tests_base.TestCase):
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
         ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
         self.storage.delete_metric(self.metric)
+        self.storage.process_background_tasks(self.index, sync=True)
 
     def test_delete_nonempty_metric_unprocessed(self):
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
         ])
         self.storage.delete_metric(self.metric)
-
-    def test_delete_nonempty_metric_with_process(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-        ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric]
-            self.storage.process_background_tasks(self.index)
-        self.storage.delete_metric(self.metric)
+        self.storage.process_background_tasks(self.index, sync=True)
 
     def test_measures_reporting(self):
-        self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-        ])
-        self.storage.add_measures(self.metric, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
-        ])
-        report = self.storage.measures_report()
-        self.assertEqual({str(self.metric.id): 2}, report)
-
-        metric2 = storage.Metric(uuid.uuid4(), self.archive_policies['low'])
-        self.storage.add_measures(metric2, [
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
-            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
-        ])
-        report = self.storage.measures_report()
-        self.assertEqual({str(self.metric.id): 2, str(metric2.id): 1}, report)
-
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric, metric2]
-            self.storage.process_measures(self.index)
-
-        report = self.storage.measures_report()
-        self.assertEqual({}, report)
+        report = self.storage.measures_report(True)
+        self.assertIsInstance(report, dict)
+        self.assertIn('summary', report)
+        self.assertIn('metrics', report['summary'])
+        self.assertIn('measures', report['summary'])
+        self.assertIn('details', report)
+        self.assertIsInstance(report['details'], dict)
+        report = self.storage.measures_report(False)
+        self.assertIsInstance(report, dict)
+        self.assertIn('summary', report)
+        self.assertIn('metrics', report['summary'])
+        self.assertIn('measures', report['summary'])
+        self.assertNotIn('details', report)
 
     def test_add_measures_big(self):
-        m = storage.Metric(uuid.uuid4(), self.archive_policies['high'])
+        m, __ = self._create_metric('high')
         self.storage.add_measures(m, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, i, j), 100)
             for i in six.moves.range(0, 60) for j in six.moves.range(0, 60)])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [m]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
 
         self.assertEqual(3661, len(self.storage.get_measures(m)))
 
+    @mock.patch('gnocchi.carbonara.AggregatedTimeSerie.POINTS_PER_SPLIT', 48)
+    def test_add_measures_big_update_subset(self):
+        m, m_sql = self._create_metric('medium')
+        measures = [
+            storage.Measure(datetime.datetime(2014, 1, i, j, 0, 0), 100)
+            for i in six.moves.range(1, 6) for j in six.moves.range(0, 24)]
+        measures.append(
+            storage.Measure(datetime.datetime(2014, 1, 6, 0, 0, 0), 100))
+        self.storage.add_measures(m, measures)
+        self.storage.process_background_tasks(self.index, sync=True)
+
+        self.storage.add_measures(m, [
+            storage.Measure(datetime.datetime(2014, 1, 6, 1, 0, 0), 100)])
+
+        with mock.patch.object(self.storage, '_store_metric_measures') as c:
+            self.storage.process_background_tasks(self.index, sync=True)
+        count = 0
+        for call in c.mock_calls:
+            if mock.call(m_sql, mock.ANY, 'mean', 3600.0, mock.ANY) == call:
+                count += 1
+        self.assertEqual(1, count)
+
+    def test_delete_old_measures(self):
+        self.storage.add_measures(self.metric, [
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+        ])
+        self.storage.process_background_tasks(self.index, sync=True)
+
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12), 300.0, 69.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 5), 300.0, 23.0),
+            (utils.datetime_utc(2014, 1, 1, 12, 10), 300.0, 44.0),
+        ], self.storage.get_measures(self.metric))
+
+        # One year later…
+        self.storage.add_measures(self.metric, [
+            storage.Measure(datetime.datetime(2015, 1, 1, 12, 0, 1), 69),
+        ])
+        self.storage.process_background_tasks(self.index, sync=True)
+
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
+            (utils.datetime_utc(2015, 1, 1), 86400.0, 69),
+            (utils.datetime_utc(2015, 1, 1, 12), 3600.0, 69),
+            (utils.datetime_utc(2015, 1, 1, 12), 300.0, 69),
+        ], self.storage.get_measures(self.metric))
+
     def test_add_and_get_measures(self):
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
@@ -144,9 +170,7 @@ class TestStorageDriver(tests_base.TestCase):
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
         ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
@@ -157,6 +181,8 @@ class TestStorageDriver(tests_base.TestCase):
         ], self.storage.get_measures(self.metric))
 
         self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 39.75),
             (utils.datetime_utc(2014, 1, 1, 12, 10), 300.0, 44.0),
         ], self.storage.get_measures(
             self.metric,
@@ -171,14 +197,17 @@ class TestStorageDriver(tests_base.TestCase):
             self.metric,
             to_timestamp=datetime.datetime(2014, 1, 1, 12, 6, 0)))
 
-        self.assertEqual(
-            [],
-            self.storage.get_measures(
-                self.metric,
-                to_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 10),
-                from_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 10)))
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12, 10), 300.0, 44.0),
+        ], self.storage.get_measures(
+            self.metric,
+            to_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 10),
+            from_timestamp=datetime.datetime(2014, 1, 1, 12, 10, 10)))
 
         self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
             (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 39.75),
             (utils.datetime_utc(2014, 1, 1, 12), 300.0, 69.0),
         ], self.storage.get_measures(
@@ -186,13 +215,22 @@ class TestStorageDriver(tests_base.TestCase):
             from_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 0),
             to_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 2)))
 
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 39.75),
+            (utils.datetime_utc(2014, 1, 1, 12), 300.0, 69.0),
+        ], self.storage.get_measures(
+            self.metric,
+            from_timestamp=timeutils.parse_isotime("2014-1-1 13:00:00+01:00"),
+            to_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 2)))
+
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 39.75),
         ], self.storage.get_measures(
             self.metric,
             from_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 0),
             to_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 2),
-            granularity=3600))
+            granularity=3600.0))
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1, 12), 300.0, 69.0),
@@ -200,10 +238,12 @@ class TestStorageDriver(tests_base.TestCase):
             self.metric,
             from_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 0),
             to_timestamp=datetime.datetime(2014, 1, 1, 12, 0, 2),
-            granularity=300))
+            granularity=300.0))
 
-        self.assertEqual([], self.storage.get_measures(self.metric,
-                                                       granularity=42))
+        self.assertRaises(storage.GranularityDoesNotExist,
+                          self.storage.get_measures,
+                          self.metric,
+                          granularity=42)
 
     def test_get_cross_metric_measures_unknown_metric(self):
         self.assertEqual([],
@@ -244,6 +284,26 @@ class TestStorageDriver(tests_base.TestCase):
                           [self.metric, metric2],
                           aggregation='last')
 
+    def test_get_cross_metric_measures_unknown_granularity(self):
+        metric2 = storage.Metric(uuid.uuid4(),
+                                 self.archive_policies['low'])
+        self.storage.add_measures(self.metric, [
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+        ])
+        self.storage.add_measures(metric2, [
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 4),
+            storage.Measure(datetime.datetime(2014, 1, 1, 12, 12, 45), 44),
+        ])
+        self.assertRaises(storage.GranularityDoesNotExist,
+                          self.storage.get_cross_metric_measures,
+                          [self.metric, metric2],
+                          granularity=12345.456)
+
     def test_add_and_get_cross_metric_measures_different_archives(self):
         metric2 = storage.Metric(uuid.uuid4(),
                                  self.archive_policies['no_granularity_match'])
@@ -265,8 +325,7 @@ class TestStorageDriver(tests_base.TestCase):
                           [self.metric, metric2])
 
     def test_add_and_get_cross_metric_measures(self):
-        metric2 = storage.Metric(uuid.uuid4(),
-                                 self.archive_policies['low'])
+        metric2, __ = self._create_metric()
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
@@ -279,9 +338,7 @@ class TestStorageDriver(tests_base.TestCase):
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 10, 31), 4),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 13, 10), 4),
         ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric, metric2]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
 
         values = self.storage.get_cross_metric_measures([self.metric, metric2])
         self.assertEqual([
@@ -293,13 +350,17 @@ class TestStorageDriver(tests_base.TestCase):
         ], values)
 
         values = self.storage.get_cross_metric_measures(
-            [self.metric, metric2], from_timestamp='2014-01-01 12:10:00')
+            [self.metric, metric2],
+            from_timestamp=utils.to_timestamp('2014-01-01 12:10:00'))
         self.assertEqual([
-            (utils.datetime_utc(2014, 1, 1, 12, 10, 0), 300.0, 24.0)
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 22.25),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 22.25),
+            (utils.datetime_utc(2014, 1, 1, 12, 10, 0), 300.0, 24.0),
         ], values)
 
         values = self.storage.get_cross_metric_measures(
-            [self.metric, metric2], to_timestamp='2014-01-01 12:05:00')
+            [self.metric, metric2],
+            to_timestamp=utils.to_timestamp('2014-01-01 12:05:00'))
 
         self.assertEqual([
             (utils.datetime_utc(2014, 1, 1, 0, 0, 0), 86400.0, 22.25),
@@ -309,23 +370,37 @@ class TestStorageDriver(tests_base.TestCase):
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            to_timestamp='2014-01-01 12:10:10',
-            from_timestamp='2014-01-01 12:10:10')
-        self.assertEqual([], values)
+            to_timestamp=utils.to_timestamp('2014-01-01 12:10:10'),
+            from_timestamp=utils.to_timestamp('2014-01-01 12:10:10'))
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 22.25),
+            (utils.datetime_utc(2014, 1, 1, 12), 3600.0, 22.25),
+            (utils.datetime_utc(2014, 1, 1, 12, 10), 300.0, 24.0),
+        ], values)
 
         values = self.storage.get_cross_metric_measures(
             [self.metric, metric2],
-            from_timestamp='2014-01-01 12:00:00',
-            to_timestamp='2014-01-01 12:00:01')
+            from_timestamp=utils.to_timestamp('2014-01-01 12:00:00'),
+            to_timestamp=utils.to_timestamp('2014-01-01 12:00:01'))
 
         self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1), 86400.0, 22.25),
             (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 3600.0, 22.25),
             (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 300.0, 39.0),
         ], values)
 
+        values = self.storage.get_cross_metric_measures(
+            [self.metric, metric2],
+            from_timestamp=utils.to_timestamp('2014-01-01 12:00:00'),
+            to_timestamp=utils.to_timestamp('2014-01-01 12:00:01'),
+            granularity=300.0)
+
+        self.assertEqual([
+            (utils.datetime_utc(2014, 1, 1, 12, 0, 0), 300.0, 39.0),
+        ], values)
+
     def test_add_and_get_cross_metric_measures_with_holes(self):
-        metric2 = storage.Metric(uuid.uuid4(),
-                                 self.archive_policies['low'])
+        metric2, __ = self._create_metric()
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1), 69),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
@@ -339,9 +414,7 @@ class TestStorageDriver(tests_base.TestCase):
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 6),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 13, 10), 2),
         ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric, metric2]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
 
         values = self.storage.get_cross_metric_measures([self.metric, metric2])
         self.assertEqual([
@@ -353,8 +426,7 @@ class TestStorageDriver(tests_base.TestCase):
         ], values)
 
     def test_search_value(self):
-        metric2 = storage.Metric(uuid.uuid4(),
-                                 self.archive_policies['low'])
+        metric2, __ = self._create_metric()
         self.storage.add_measures(self.metric, [
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 0, 1,), 69),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 7, 31), 42),
@@ -369,16 +441,18 @@ class TestStorageDriver(tests_base.TestCase):
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 9, 31), 6),
             storage.Measure(datetime.datetime(2014, 1, 1, 12, 13, 10), 2),
         ])
-        with mock.patch.object(self.index, 'get_metrics') as f:
-            f.return_value = [self.metric, metric2]
-            self.storage.process_background_tasks(self.index, True)
+        self.storage.process_background_tasks(self.index, sync=True)
 
         self.assertEqual(
             {metric2: [],
-             self.metric: [(utils.datetime_utc(2014, 1, 1, 12), 300, 69)]},
+             self.metric: [
+                 (utils.datetime_utc(2014, 1, 1), 86400, 33),
+                 (utils.datetime_utc(2014, 1, 1, 12), 3600, 33),
+                 (utils.datetime_utc(2014, 1, 1, 12), 300, 69),
+                 (utils.datetime_utc(2014, 1, 1, 12, 10), 300, 42)]},
             self.storage.search_value(
                 [metric2, self.metric],
-                {u"≥": 50}))
+                {u"≥": 30}))
 
         self.assertEqual(
             {metric2: [], self.metric: []},
@@ -387,3 +461,63 @@ class TestStorageDriver(tests_base.TestCase):
                 {u"∧": [
                     {u"eq": 100},
                     {u"≠": 50}]}))
+
+
+class TestMeasureQuery(base.BaseTestCase):
+    def test_equal(self):
+        q = storage.MeasureQuery({"=": 4})
+        self.assertTrue(q(4))
+        self.assertFalse(q(40))
+
+    def test_gt(self):
+        q = storage.MeasureQuery({">": 4})
+        self.assertTrue(q(40))
+        self.assertFalse(q(4))
+
+    def test_and(self):
+        q = storage.MeasureQuery({"and": [{">": 4}, {"<": 10}]})
+        self.assertTrue(q(5))
+        self.assertFalse(q(40))
+        self.assertFalse(q(1))
+
+    def test_or(self):
+        q = storage.MeasureQuery({"or": [{"=": 4}, {"=": 10}]})
+        self.assertTrue(q(4))
+        self.assertTrue(q(10))
+        self.assertFalse(q(-1))
+
+    def test_modulo(self):
+        q = storage.MeasureQuery({"=": [{"%": 5}, 0]})
+        self.assertTrue(q(5))
+        self.assertTrue(q(10))
+        self.assertFalse(q(-1))
+        self.assertFalse(q(6))
+
+    def test_math(self):
+        q = storage.MeasureQuery(
+            {
+                u"and": [
+                    # v+5 is bigger 0
+                    {u"≥": [{u"+": 5}, 0]},
+                    # v-6 is not 5
+                    {u"≠": [5, {u"-": 6}]},
+                ],
+            }
+        )
+        self.assertTrue(q(5))
+        self.assertTrue(q(10))
+        self.assertFalse(q(11))
+
+    def test_empty(self):
+        q = storage.MeasureQuery({})
+        self.assertFalse(q(5))
+        self.assertFalse(q(10))
+
+    def test_bad_format(self):
+        self.assertRaises(storage.InvalidQuery,
+                          storage.MeasureQuery,
+                          {"foo": [{"=": 4}, {"=": 10}]})
+
+        self.assertRaises(storage.InvalidQuery,
+                          storage.MeasureQuery,
+                          {"=": [1, 2, 3]})
